function[prob, params] = hec_problem
% Hand-Eye calibration, quaternion parameterization
% Jan Heller, Didier Henrion, Tomas Pajdla: 
% "Hand-Eye and Robot-World Calibration by Global Polynomial Optimization", 
% 2014 IEEE International Conference on Robotics and Automation (ICRA 2014), Hong Kong, China

syms qx1 qx2 qx3 qx4 tx1 tx2 tx3 real;
syms ra1 ra2 ra3 ra4 ra5 ra6 ra7 ra8 ra9 ta1 ta2 ta3 real;
syms rb1 rb2 rb3 rb4 rb5 rb6 rb7 rb8 rb9 tb1 tb2 tb3 real;
syms n real

X = getX(qx1, qx2, qx3, qx4, tx1, tx2, tx3);

A = sym(eye(4,4));
A(1:3, 1:3) = [ra1, ra2, ra3; ra4, ra5, ra6; ra7, ra8, ra9];
A(1:3, 4) = [ta1; ta2; ta3];

B = sym(eye(4,4));
B(1:3,1:3) = [rb1, rb2, rb3; rb4, rb5, rb6; rb7, rb8, rb9];
B(1:3, 4) = [tb1; tb2; tb3];

E = A*X - X*B;

prob.obj = dot(E(:), E(:));
prob.vars = [qx1 qx2 qx3 qx4 tx1 tx2 tx3];
prob.cons = [qx1^2 + qx2^2 + qx3^2 + qx4^2 == 1, ...
             qx1 >= 0, ...
             tx1^2 + tx2^2 + tx3^2 <= n];
         
prob.rpars = [ra1 ra4 ra7 ra2 ra5 ra8 ra3 ra6 ra9 ta1 ta2 ta3 ...
              rb1 rb4 rb7 rb2 rb5 rb8 rb3 rb6 rb9 tb1 tb2 tb3 ];          
prob.ppars = n;          
          
params.classname = 'HecProblem';         
params.filename = 'hec_problem';
params.relax_order = 2;
end

function X = getX(a, b, c, d, t1, t2, t3)
  X = [a^2 + b^2 - c^2 - d^2,         2*b*c - 2*a*d,         2*b*d + 2*a*c, t1;
               2*b*c + 2*a*d, a^2 - b^2 + c^2 - d^2,         2*c*d - 2*a*b, t2;
               2*b*d - 2*a*c,         2*c*d + 2*a*b, a^2 - b^2 - c^2 + d^2, t3;
                           0,                     0,                     0, 1];
end
