function[prob, params] = pnp_problem
% PnP problem for generalized camera model
% Gerald Schweighofer and Axel Pinz:
% "Globally Optimal O(n) Solution to the PnP Problem for General Camera Models",
% Proceedings of the British Machine Vision Conference 2008, Leeds

q = sym('q', [4, 1]);
assume(q, 'real');
X = sym('X', [3, 1]);
assume(X, 'real');
v = sym('v', [3, 1]);
assume(v, 'real');
c = sym('c', [3, 1]);
assume(c, 'real');
T = sym('T', [3, 10]);
assume(T, 'real');

z13 = zeros(1, 3);
CX = [X', z13, z13; z13, X', z13; z13, z13, X'];
Q = eye(3,3) - v * v';

R = q2rot(q);
r = R(:);

M = ([CX c] + T)' * Q * ([CX c] + T);
Mr = M(1:(end-1), 1:(end-1));
Mc = 2*M(1:(end-1), end);
Mcc = M(end, end);

prob.obj = r' * Mr * r + Mc' * r + Mcc;
prob.vars = q;
prob.cons = [q(1)^2 + q(2)^2 + q(3)^2 + q(4)^2 == 1, ...
             q(1) >= 0, ...
             ];
         
prob.rpars = [v' c' X'];          
prob.ppars = T(:)';
          
params.classname = 'PnpProblem';         
params.filename = 'pnp_problem';
params.relax_order = 2;

end

function R = q2rot(q)
  R = [q(1)^2 + q(2)^2 - q(3)^2 - q(4)^2,         2*q(2)*q(3) - 2*q(1)*q(4),         2*q(2)*q(4) + 2*q(1)*q(3);
               2*q(2)*q(3) + 2*q(1)*q(4), q(1)^2 - q(2)^2 + q(3)^2 - q(4)^2,         2*q(3)*q(4) - 2*q(1)*q(2);
               2*q(2)*q(4) - 2*q(1)*q(3),         2*q(3)*q(4) + 2*q(1)*q(2), q(1)^2 - q(2)^2 - q(3)^2 + q(4)^2];           
end