#include <iostream>
#include <stdexcept>

#include <Eigen/Dense>

#ifdef CSDP_SOLVER
#include <gposolver/gposolver_csdp.h>
#elif MOSEK_SOLVER
#include <gposolver/gposolver_mosek.h>
#elif SDPA_SOLVER
#include <gposolver/gposolver_sdpa.h>
#else
#error Define CSDP_SOLVER, MOSEK_SOLVER, or SDPA_SOLVER macros
#endif

#include "pop_problem.h"

using namespace GpoSolver;

int main(int argc, const char* argv[]) {
#ifdef CSDP_SOLVER
  GpoSolverCsdp<PopProblem> gposolver;
#elif MOSEK_SOLVER
  GpoSolverMosek<PopProblem> gposolver;
#elif SDPA_SOLVER
  GpoSolverSdpa<PopProblem> gposolver;
#endif

  try
    {
      GpoSolverStatus status;
      GpoSolverSolutions sols;

      double rvals[][4] = {{1, 2, 2, 3}, {2, 1, 1, 2}};
      double pvals[] = {1, 2, 3, 4};

      // Call GpoSolver
      status = gposolver.solve(2, (double *) rvals, (double *) pvals, sols);

      cout << "GpoSolver: " << sols.size() << " solution(s) extracted" << endl;

      if (sols.size() == 0)
          return 0;
      else if (status == SUCCESS_GLOPT)
        cout << "Global optimality certified numerically" << endl;
      else if (status == SUCCESS)
        cout << "Alas, global optimality could not be certified" << endl;

      int c = 1;
      for (GpoSolverSolutions::iterator it = sols.begin(); it != sols.end(); it++)
        {
          std::vector<double> &sol = *(it);
          cout << "Solution " << c++ << ": ";
	  for (int i = 0; i < sol.size(); i++)
            cout << sol[i] << " ";
	  cout << endl;
        }
    }
  catch (exception& e)
    {
      cerr << "GpoSolver exception: " << e.what() << endl;
      return 1;
    }

  return 0;
}
